#include <limits.h>
#include <math.h>
#include "SafeArray.h"
#include "defs.h"
#include "utils.h"
#include "mfmodel.h"
#include "mf_updpost.h"
#include "shmem.h"
#include "mf_priors.h"
#include "mex.h"
#include "dp.h"
#include "randlib.h"
#include "NormalIGamma.h"

#include <time.h>

numtype t0, t1, t3, tA, tMG, tInj, tZ, tMH, tLogP1, t0Prop, t0PropUP, tPriorMu, tDPEStep, tDPLogP, tGibbs, tRnd, nRnd;

TMemoryUpdPost sm;

//**************************************************************
//**************************************************************
void MFInvGammaUpdatePosterior (
					  numtype *PosteriorA,
					  numtype *PosteriorB, 
			    const numtype *Data, 
				const numtype *NRepl, 
				const numtype *Mu, 
				const numtype *Dims, 
				const numtype *PriorA, 
				const numtype *PriorB,
				const int D,
				const int NDims,
				const int DNMaxRepl)
{
	int NMaxRepl = DNMaxRepl/D;

	// sample new ReplVar value given mean values, configuration and the prior
	int h,t, r;
	numtype NObs, ObservedSumSq, diff;
    
    // Accumulate observed values from all time points in a given bucket for replicate variance estimation
    NObs = 0.; ObservedSumSq = 0.;
    for (h = 0; h < NDims; h++)
	{
        t = Dims[h]-1;
        for (r = 0; r < NRepl[t]; r++)
		{
			if (Data[r + t*NMaxRepl] != -Inf)
			{
				diff = Data[r + t*NMaxRepl]-Mu[t];
				ObservedSumSq = ObservedSumSq + diff*diff;
				NObs = NObs + 1;
			}
		}
     }
        
    *PosteriorA = (*PriorA) + 0.5*NObs;
    *PosteriorB = 1.0 / ((1.0/(*PriorB)) + 0.5*ObservedSumSq);

}

void MFInvGammaUpdatePosteriorBatch (
					  numtype *ReplVar,
					  numtype *Posterior,
			    const numtype *Data, 
				const numtype *NRepl, 
				const numtype *Mu, 
				const numtype *ReplVarOld, 
				const numtype *ReplVarGroups, 
				const numtype *ReplVarGroupLen, 
				const numtype *PriorA, 
				const numtype *PriorB,
				const int D,
				const int NReplVarGroups,
				const int NMaxGroupLen,
				const int DNMaxRepl)
{

	SafeArray<const numtype> ReplVarGroupLen_	(ReplVarGroupLen, NReplVarGroups);
	SafeArray<const numtype> ReplVarGroups_		(ReplVarGroups, NReplVarGroups, NMaxGroupLen);

	SafeArray<const numtype> ReplVarOld_(ReplVarOld, D);

	SafeArray<numtype> ReplVar_	(ReplVar, D);
	SafeArray<numtype> Posterior_(Posterior, NReplVarGroups,2);
	
	int i,j;

	numtype *ReplVarGroupCur = new numtype[NMaxGroupLen];
	numtype PosteriorA, PosteriorB, tmp;

	for (i = 1; i <= D; i++)
		ReplVar_(i) = ReplVarOld_(i);

	for (i = 1; i <= NReplVarGroups; i++)
	{
		// create current config 
		SafeArray<numtype> ReplVarGroupCur_(ReplVarGroupCur, ReplVarGroupLen_(i));
		for (j = 1; j <= ReplVarGroupLen_(i); j++)
		{
			ReplVarGroupCur_(j) = ReplVarGroups_(i,j);
		}

		// get posterior params for current config
		MFInvGammaUpdatePosterior (&PosteriorA, &PosteriorB,Data, NRepl, Mu, ReplVarGroupCur, PriorA, PriorB,D,ReplVarGroupLen_(i),DNMaxRepl);

		Posterior_(i,1) = PosteriorA;
		Posterior_(i,2) = PosteriorB;

		// sample from inverse gamma
		gamrnd_c(&tmp, PosteriorA, PosteriorB,1,1);
		tmp = 1./tmp;
		for (j = 1; j <= ReplVarGroupLen_(i); j++)
		{
			ReplVar_(ReplVarGroupCur_(j)) = tmp;
		}

	}

	delete[] ReplVarGroupCur;

}

void MFNormalFVUpdatePosteriorInv (
					  numtype *PosteriorMean,
					  numtype *PosteriorVar, 
			    const numtype *Observed, 
				const numtype *FixedInvVar, 
				const numtype *NObserved,
				const numtype *PriorMean, 
				const numtype *PriorInvVar, 
				const int NMaxObserved,
				const int D,
				const int NFixedVar)
{
	SafeArray<const numtype> Observed_		(Observed, NMaxObserved, D, NFixedVar);
	SafeArray<const numtype> FixedInvVar_	(FixedInvVar, D, NFixedVar);
	
    numtype PosteriorPrecision;

	char err[1024];
	int i, h, j;
	numtype s;
    for (j = 0; j < D; j++)
	{
		PosteriorPrecision = PriorInvVar[j];
		PosteriorMean[j] = PriorMean[j]*PriorInvVar[j];

		for (i = 1; i <= NFixedVar; i++)
		{
			PosteriorPrecision += NObserved[i-1]*FixedInvVar_(j+1,i);
			s = 0;
			for (h = 1; h <= NObserved[i-1]; h++)
			{
				s += Observed_(h,j+1,i);
			}
			PosteriorMean[j] += s*FixedInvVar_(j+1,i);
		}

		PosteriorMean[j] /= PosteriorPrecision;
		PosteriorVar[j] = 1./PosteriorPrecision;
    }
}

void MFNormalFVUpdatePosterior (
					  numtype *PosteriorMean,
					  numtype *PosteriorVar, 
			    const numtype *Observed, 
				const numtype *FixedVar, 
				const numtype *NObserved,
				const numtype *PriorMean, 
				const numtype *PriorVar, 
				const int NMaxObserved,
				const int D,
				const int NFixedVar)
{

	SafeArray<const numtype> Observed_		(Observed, NMaxObserved, D, NFixedVar);
	SafeArray<const numtype> FixedVar_	(FixedVar, D, NFixedVar);
	
    numtype PosteriorPrecision;

	char err[1024];
	int i, h, j;
	numtype s;
    for (j = 0; j < D; j++)
	{
		PosteriorPrecision = 1/PriorVar[j];
		PosteriorMean[j] = PriorMean[j]/PriorVar[j];

		for (i = 1; i <= NFixedVar; i++)
		{
			PosteriorPrecision += NObserved[i-1]/FixedVar_(j+1,i);
			s = 0;
			for (h = 1; h <= NObserved[i-1]; h++)
			{
				s += Observed_(h,j+1,i);
			}
			PosteriorMean[j] += s/FixedVar_(j+1,i);
		}

		PosteriorMean[j] /= PosteriorPrecision;
		PosteriorVar[j] = 1./PosteriorPrecision;
     }
}

void MFNormalFVUpdatePosterior1 (
					  numtype *PosteriorMean,
					  numtype *PosteriorVar, 
			    const numtype *Observed, 
				const numtype FixedVar, 
				const numtype NObserved,
				const numtype PriorMean, 
				const numtype PriorVar)
{

    numtype PosteriorPrecision;

	char err[1024];
	int h;

	PosteriorPrecision = 1/PriorVar;
	*PosteriorMean = PriorMean/PriorVar;

	PosteriorPrecision += NObserved/FixedVar;

	numtype s = 0;
	for (h = 0; h < NObserved; h++)
	{
		s += Observed[h];
	}
	*PosteriorMean += s/FixedVar;

	*PosteriorMean /= PosteriorPrecision;
	*PosteriorVar = 1./PosteriorPrecision;
}

void MFNormalFVKUpdatePosterior (
					  numtype *PosteriorMean,
					  numtype *PosteriorVar, 
			    const numtype *Observed, 
				const numtype FixedVar, 
				const numtype KFactor,
				const numtype PriorMean, 
				const numtype PriorVar, 
				const int NObserved)
{
	numtype PosteriorPrecision;
	
	int h;

	PosteriorPrecision = 1/PriorVar;
	*PosteriorMean = PriorMean/PriorVar;

	PosteriorPrecision += KFactor*KFactor*NObserved/FixedVar;
	for (h = 0; h < NObserved; h++)
	{
		*PosteriorMean += KFactor*Observed[h]/FixedVar;
	}
	*PosteriorMean /= PosteriorPrecision;
	*PosteriorVar = 1./PosteriorPrecision;
}

void MFNormalFVKVUpdatePosterior (
					  numtype *PosteriorMean,
					  numtype *PosteriorVar, 
			    const numtype *Observed, 
				const numtype FixedVar, 
				const numtype KFactor,
				const numtype PriorMean, 
				const numtype PriorVar, 
				const int NObserved)
{
    numtype PosteriorPrecision;
	
	int h;

	PosteriorPrecision = 1/PriorVar;
	*PosteriorMean = PriorMean/PriorVar;

	PosteriorPrecision += NObserved/FixedVar;
	for (h = 0; h < NObserved; h++)
	{
		*PosteriorMean += Observed[h]/(KFactor*FixedVar);
	}

	*PosteriorMean /= PosteriorPrecision;
	*PosteriorVar = 1./PosteriorPrecision;
}


