function [HPriors, pKGivenY, LogPY] = estimatePeriodicity (Data, Config, pvaluePR, pvalueDE)
% Estimate parameters for 3 components from the Data: 
% background(BG), differential expression (DE), and periodicity (PR);
% use a 3-component mixture to derive posterior probabilities of BG / DE /
% PR components
%
% Input parameters
% Data               = observations
%   NRepl            - a matrix of size NxD, with NRepl[i,j] being number
%                      of replicates for gene i, time point j;
%                      total number of genes is N; 
%                      total number of time points is D
%   v                - a matrix of size Nx(D*NMaxRepl) with observation
%                      v[i,(j-1)*NMaxRepl + r] being replicate r (1 <= r <= NRepl[i,j]) of time
%                      point j (1 <= j <= D), gene i (1 <= i <= N)
%   L                - vector of length N; all values equal to 1
%   R                - vector of length N; all values equal to D
%   x                - optional vector of length D specifying time points
%
% Config             = Configuration specifying structure of periodicity
%   NCycleDep        - a matrix of length L (number of time points in the
%                      ideal cycle)
%   CycleDep
%
% pvaluePR           - p value for initial assignment of genes to the
%                      periodic component based on ANOVA detector
% pvalueDE           - p value for initial assignment of genes to the 
%                      DE component based on ANOVA detector

% Return values:
% HPriors: estimated component parameters
%   BG: Mu0, K0, A1, A2 : mean / scale / df / inverse scale of NIG prior
%                         for BG component
%   DE: Mu0, K0, A1, A2 : mean / scale / df / inverse scale of NIG prior
%                         for DE component
%   PR: Mu0, K0, A1, A2 : mean / scale / df / inverse scale of NIG prior
%                         for PR component
%
% pKGivenY: a matrix of size N x 3 with component probabilities for each
%           of N observations in Data
%   pKGivenY[i,1] - probability of PR component for observation i
%   pKGivenY[i,2] - probability of DE component for observation i
%   pKGivenY[i,3] - probability of BG component for observation i
%
% LogPY: a vector of length N with marginal likelihood for each of N genes
%        

[DataMatr, L, R, Len, N, D, Q, X, NRepl, NMaxRepl] = GetDataLRLen(Data);
[M, V] = GetReplVar2(Data);

% estimate HPriors

% 1a. use ANOVA to detect tentative DE / PR / BG genes
pPR = GetANOVAPeriodic(Data, Config);
[dummy, ordPR] = sort(pPR);
pDE = GetANOVADE(Data);
[dummy, ordDE] = sort(pDE);

% 1b. preliminary sets of periodic / DE / BG genes for fitting
% empirical priors for variance
idxClass.PR = find(pPR < pvaluePR); 
idxClass.DE = find(pDE < pvalueDE); % idxClass.DE = setdiff(idxClass.DE, idxClass.PR);
idxClass.BG = find(pDE > pvalueDE); % idxClass.BG = setdiff([1:N], union(idxClass.PR, idxClass.DE));

% 1c. estimate prior component probabilities
pComp = [length(idxClass.PR), length(idxClass.DE), length(idxClass.BG)];
pComp = pComp / sum(pComp);

% 2. estimate HPriors given tentative assignment to DE / PR / BG classes
BConjugateModel = 1;
BEstimateMu0 = 0;
HPriors = SPMLEDefineHPriors (Data, Config, BConjugateModel, BEstimateMu0, pComp, -1, -1, idxClass);

% 3. run detection using these priors
[LogPY, pKGivenY, LogPYGivenK, Theta.PR, Theta.DE, Theta.BG] = ...
    GetSPLogP(DataMatr, NRepl, Config.CycleDep, Config.NCycleDep', HPriors.PComp', ...
    HPriors.PR.Mu0', HPriors.PR.K0', HPriors.PR.A1', HPriors.PR.A2', ...
    HPriors.DE.Mu0', HPriors.DE.K0', HPriors.DE.A1', HPriors.DE.A2', ...
    HPriors.BG.Mu0(1), HPriors.BG.K0(1), HPriors.BG.A1(1), HPriors.BG.A2(1));

% 4. output log-likelihood per point
sum(LogPY)/sum(sum(Data.NRepl))

